import dotenv from 'dotenv';
import {Client, IntentsBitField} from 'discord.js';
import sqlite3 from 'sqlite3';

const db = new sqlite3.Database("./db/employeescoreboard.db", sqlite3.OPEN_READWRITE,(err) => {
    if (err) return console.error(err.message);
});

//Id of the channel where the gear scoreboard will be posted to, to post in the testing channel it is 1306669146804715693

dotenv.config();

const client = new Client({
    intents: [
        IntentsBitField.Flags.Guilds,
        IntentsBitField.Flags.GuildMembers,
        IntentsBitField.Flags.GuildMessages,
        IntentsBitField.Flags.MessageContent,
    ]
});


client.on('ready', (c) => {
    console.log('The Swog is prepared.');
})
//Chat commands are handled here
client.on('interactionCreate', async(interaction) => {
    if (!interaction.isChatInputCommand()) return;
    
    if (interaction.commandName === 'croak') {
        interaction.reply('Croak!');
    }

    if (interaction.commandName === 'gear') {
        const addRemove = interaction.options.get('addremove').value;
        const gearType = interaction.options.get('geartype').value;
        const employee = interaction.options.get('employee').value;
        if ( addRemove == 'add') {
            AddGear(gearType, employee)
            interaction.reply(`Added a gear to ${employee}`)
        };

    }

    if (interaction.commandName === 'employee') {
        const hireFirePromote = interaction.options.get('hire-fire-promote').value;
        const employeeName = interaction.options.get('employee-name').value;
        if (hireFirePromote == 'hire') {
            CreateEmployee(employeeName);
            interaction.reply(`Hired ${employeeName}!`);
        }
        if (hireFirePromote == 'fire') {
            DeleteEmployee(employeeName);
        }
    }

    if (interaction.commandName === 'promote') {
        const employeeName = interaction.options.get('employee-name').value;
        const rank = interaction.options.get('rank').value;
        PromoteEmployee(employeeName, rank);
        interaction.reply(`Changed ${employeeName}\'s rank to ${rank}`);
    }

    if (interaction.commandName === 'scoreboard') {
        const updateOrDisplay = interaction.options.get('update-display').value;
        if (updateOrDisplay == 'display') {
           
            const gearchannel = client.channels.cache.get('1306669146804715693');
            let scoreBoardMessage = await BuildScoreboard();
            console.log("Scoreboard goes here: " + scoreBoardMessage);
            if (scoreBoardMessage != "") {
                gearchannel.send(scoreBoardMessage);
                interaction.reply("Displaying Scoreboard");
            }
            else {
                console.log("Something went wrong when building the scoreboard.");
            };
        };
    };

});

function CreateEmployee(Name) {
    db.run(`INSERT INTO Employees (name) VALUES (\'${Name}\')`);
};
function DeleteEmployee(Name) {
    db.run(`DELETE FROM Employees WHERE name = \'${Name}\'`);
};
function PromoteEmployee(Name, rank) {
    db.run(`UPDATE Employees SET rank = \'${rank}\' WHERE name = \'${Name}\'`)
};

function AddGear(Type,Employee) {
    if (Type != "coppergear" | "silvergear" | "goldgear") return
    db.run(`UPDATE Employees SET ${Type} = ${Type} + 1 WHERE name = \'${Employee}\' `)
};

function BuildScoreboard() {
    return new Promise((resolve) => {
        let messageContent = "";
        db.each('SELECT name, rank, coppergear, silvergear, goldgear, purplegear FROM Employees', (err, character) => {
            if (err != null) {
                console.log(err.message);
            };
    
            let copperGearString = "";
            console.log(character.coppergear);
            for (let i = 0; i < character.coppergear; i++) {
                copperGearString += ":coppergear:" 
            };
    
            let silverGearString = "";
            console.log(character.silvergear);
            for (let i = 0; i < character.silvergear; i++) {
                silvergearstring += ":gear:"
            }

            let goldGearString = "";
            messageContent += `**${character.name}** \n ##${copperGearString} \n`;
    
        }, () => {resolve(messageContent); });
    });
    
};

client.login(process.env.DISCORD_TOKEN);